#include <HALOptions.h>
#include "BluntServer.h"
#include "BluntClient.h"

extern "C" Ref MStart (RefArg rcvr, RefArg location, RefArg driver)
{
	BluntServer* server = NULL;
	BluntClient* client = NULL;
	UChar* pc;
	ULong l;
	
	printf("----------------------------------------------------------\n");
	WITH_LOCKED_BINARY(location, p);
		pc = (UChar *) p; l = (pc[1] << 24) + (pc[3] << 16) + (pc[5] << 8) + pc[7];
	END_WITH_LOCKED_BINARY(location);

	server = new BluntServer ();
	if (server->Initialize (l, RefToInt(driver)) == noErr) {
		server->StartTask ();
		SetFrameSlot (rcvr, SYM (server), MakeInt ((ULong) server));
		client = new BluntClient (rcvr, BluntServer::Port ());
		SetFrameSlot (rcvr, SYM (client), MakeInt ((ULong) client));
		return TRUEREF;
	} else {
		delete server;
		return NILREF;
	}
}

extern "C" Ref MReset (RefArg rcvr, RefArg name)
{
	Char buffer[64];
	Ref asciiName;
	
	BluntClient* client = (BluntClient*) RefToInt (GetFrameSlot(rcvr, SYM (client)));
	asciiName = ASCIIString (name);
	WITH_LOCKED_BINARY(asciiName, n)
	memset (buffer, 0, 64);
	memcpy (buffer, n, Length (asciiName));
	client->Reset (buffer);
	END_WITH_LOCKED_BINARY(asciiName)
	return NILREF;
}

extern "C" Ref MDiscover (RefArg rcvr, RefArg time, RefArg amount)
{
	BluntClient* client = (BluntClient*) RefToInt (GetFrameSlot(rcvr, SYM (client)));
	client->Discover (RefToInt (time), RefToInt (amount));
	return NILREF;
}

extern "C" Ref MCancelDiscover (RefArg rcvr)
{
	BluntClient* client = (BluntClient*) RefToInt (GetFrameSlot(rcvr, SYM (client)));
	client->CancelDiscover ();
	return NILREF;
}

extern "C" Ref MNameRequest (RefArg rcvr, RefArg bdAddr, RefArg psRepMode, RefArg psMode)
{
	BluntClient* client = (BluntClient*) RefToInt (GetFrameSlot(rcvr, SYM (client)));
	WITH_LOCKED_BINARY(bdAddr, a)
	client->NameRequest ((UByte*) a, RefToInt (psRepMode), RefToInt (psMode));
	END_WITH_LOCKED_BINARY(bdAddr)
	return NILREF;
}

extern "C" Ref MGetServices (RefArg rcvr, RefArg bdAddr, RefArg psRepMode, RefArg psMode)
{
	BluntClient* client = (BluntClient*) RefToInt (GetFrameSlot(rcvr, SYM (client)));
	WITH_LOCKED_BINARY(bdAddr, a)
	client->GetServices ((UByte*) a, RefToInt (psRepMode), RefToInt (psMode));
	END_WITH_LOCKED_BINARY(bdAddr)
	return NILREF;
}

extern "C" Ref MConnect (RefArg rcvr)
{
	return NILREF;
}

extern "C" Ref MPair (RefArg rcvr, RefArg bdAddr, RefArg PIN, RefArg psRepMode, RefArg psMode)
{
	Char buffer[64];
	Ref asciiPIN;
	
	BluntClient* client = (BluntClient*) RefToInt (GetFrameSlot(rcvr, SYM (client)));
	asciiPIN = ASCIIString (PIN);
	WITH_LOCKED_BINARY(asciiPIN, p)
	memset (buffer, 0, 64);
	memcpy (buffer, p, Length (asciiPIN));
	END_WITH_LOCKED_BINARY(asciiPIN)
	WITH_LOCKED_BINARY(bdAddr, a)
	client->Pair ((UByte*) a, buffer, RefToInt (psRepMode), RefToInt (psMode));
	END_WITH_LOCKED_BINARY(bdAddr)
	return NILREF;
}

extern "C" Ref MStop (RefArg rcvr)
{
	BluntServer* server = NULL;
	BluntClient* client = NULL;
	TUPort port;
	
	client = (BluntClient*) RefToInt (GetFrameSlot(rcvr, SYM (client)));
	server = (BluntServer*) RefToInt (GetFrameSlot(rcvr, SYM (server)));
	client->Stop ();
	
	Sleep(1 * kSeconds);
	delete client;
	delete server;
	
	RemoveSlot (rcvr, SYM (client));
	RemoveSlot (rcvr, SYM (server));

	return NILREF;
}
