#ifndef __TRFCOMMLAYER_H
#define __TRFCOMMLAYER_H

class Channel;

#define CTRL_SABM	0x2f
#define CTRL_UA		0x63
#define CTRL_DM		0x0f
#define CTRL_DISC	0x43
#define CTRL_UIH	0xef

#define MPX_PN		0x20
#define MPX_PSC		0x10
#define MPX_CLD		0x30
#define MPX_Test	0x08
#define MPX_FCon	0x28
#define MPX_FCOff	0x18
#define MPX_MSC		0x38
#define MPX_NSC		0x04
#define MPX_RPN		0x24
#define MPX_RLS		0x14
#define MPX_SNC		0x34

#define RFCOMM_MTU_LEN 128

typedef enum {
	RFCOMM_IDLE,
	RFCOMM_CONNECT,
	RFCOMM_ACCEPT,
	RFCOMM_CONNECTED
} RFCOMMState;

class RFCOMM: public Handler
{
public:
	RFCOMMState				fState;
	Byte					fConnectionState;
	Byte					fPort;
	Byte					fDLCI;
	
	UByte					*fInputPacket;
	UShort					fInputPacketLength;
	
	UByte					*fOutputPacket;
	UShort					fOutputPacketLength;
	
	Boolean					fCR;
	Boolean					fPF;
	Boolean					fInitiator;
	
	Boolean					fConfigured;
	Byte					fRemoteMTU;
	Byte					fTargetPort;
	Boolean					fNegotiationComplete;
	
	UByte					fCRCTable[256];

							RFCOMM (void);
							~RFCOMM (void);
	
	void					Connect (void);
							
	Boolean					HandleData (UByte *data, ULong length);
	void					HandleTimer (void *userData);
	int						ProcessRFCOMMEvent (UByte *data);
	NewtonErr				ProcessEvent (UByte event, UByte *data, Byte DLCI);
	void					CompleteConnection (void);
	void					ConnectionError (void);

	void					SetAsynchronousBalancedMode ();
	void					UnnumberedAcknowlegdement ();
	void					DisconnectedMode ();
	void					Disconnect ();
	int						UnnumberedInformation ();

	void					SndSetAsynchronousBalancedMode ();
	void					SndUnnumberedAcknowlegdement ();
	void					SndDisconnectedMode ();
	void					SndDisconnect ();
	void					SndUnnumberedInformation (UByte *data, Short length);
	
	void					SndData (UByte *data, Short length);

	void					ProcessMultiplexerCommands (void);
	int						ProcessRFCOMMData (void);

	void					MPXParameterNegotiation (Boolean command);
	void					MPXRemotePortNegotiation (Boolean command);
	void					MPXModemStatusCommand (Boolean command);

	void					SndMPXParameterNegotiation (Byte negotiationDLCI, Byte flowControl, Short frameSize, Byte windowSize);
	void					SndMPXRemotePortNegotiation (void);
	void					SndMPXModemStatusCommand (Byte DLCI, Boolean isCommand, Boolean pf, Byte status);
	
	void					CreateCRCTable (void);
	UByte					CalculateCRC (UByte *data, Byte Length);
};

#endif