#include "RelocHack.h"
#include "CommToolImpl.h"
#include "RFCOMMTool.h"
#include "BluntServer.h"
#include "HCI.h"
#include "L2CAP.h"

TRFCOMMTool::TRFCOMMTool (unsigned long serviceId):
	TCommTool (serviceId)
{
	fLogLevel = 1;
	
	LOG ("-------------------------\nTRFCOMMTool\n");
	
	RelocVTable (__VTABLE__11TRFCOMMTool);
}

TRFCOMMTool::~TRFCOMMTool (void)
{
	LOG ("~TRFCOMMTool\n");
}

#pragma mark -

// ================================================================================
//  Comm Tool Functions
// ================================================================================

NewtonErr TRFCOMMTool::HandleRequest (TUMsgToken& msgToken, ULong msgType)
{
	NewtonErr r;
	BluntEvent* event;
	
	LOG ("TRFCOMMTool::HandleRequest %08x %08x\n", &msgToken, msgType);
	if (msgType == BLUNT_MSG_TYPE) {
		event = (BluntEvent *) fMsgData;
		LOG ("  Blunt event: %08x %08x %d\n", event, event->fOriginalEvent, event->fType);
		switch (event->fType) {
			case E_CONNECTION_COMPLETE: {
				BluntConnectionCompleteEvent *e = (BluntConnectionCompleteEvent *) event;
				ConnectComplete (e->fResult);
				if (e->fResult == noErr) {
					fHCIHandle = e->fHCIHandle;
					LOG	("  Connection complete: %04x\n", fHCIHandle);
				} else {
					LOG	("  Connection error: %d\n", e->fResult);
				}
				} break;
			case E_DISCONNECT_COMPLETE:
				TerminateComplete ();
				break;
		}
		delete event->fOriginalEvent;
		r = noErr;
	} else {
		r = TCommTool::HandleRequest (msgToken, msgType);
	}
	return r;
}

NewtonErr TRFCOMMTool::ProcessOptionStart (TOption* theOption, ULong label, ULong opcode)
{
	Char s[5];
	
	memset (s, 0, 5);
	memcpy (s, &label, 4);
	LOG ("TRFCOMMTool::ProcessOptionStart: %s\n", s);
	
	if (label == 'addr') {
		TRFCOMMAddressOption *o;
		o = (TRFCOMMAddressOption *) theOption;
		fPeerBdAddr[0] = o->fBdAddr[5]; fPeerBdAddr[1] = o->fBdAddr[4]; 
		fPeerBdAddr[2] = o->fBdAddr[3]; fPeerBdAddr[3] = o->fBdAddr[2]; 
		fPeerBdAddr[4] = o->fBdAddr[1]; fPeerBdAddr[5] = o->fBdAddr[0]; 
		fPeerRFCOMMPort = o->fPort;
		LOG ("  Peer Address: %02x:%02x:%02x:", o->fBdAddr[0], o->fBdAddr[1], o->fBdAddr[2]);
		LOG ("%02x:%02x:%02x ", o->fBdAddr[3], o->fBdAddr[4], o->fBdAddr[5]);
		LOG ("%d\n", o->fPort);
	} else if (label == 'pinc') {
		TRFCOMMPINCodeOption *o;
		o = (TRFCOMMPINCodeOption *) theOption;
		memcpy (fPINCode, o->fPINCode, sizeof (fPINCode));
		fPINCode[o->fPINCodeLength] = 0;
		LOG ("  PIN Code: %s\n", fPINCode);
	}
	return TCommTool::ProcessOptionStart (theOption, label, opcode);
}

void TRFCOMMTool::BindStart ()
{
	TUNameServer nameServer;
	ULong id;
	ULong spec;
	LOG ("TRFCOMMTool::BindStart\n");
	TObjectId myPort;
	
	nameServer.Lookup ("BluntServer", "TUPort", &id, &spec);
	fServerPort = id;
	myPort = * (TObjectId*) ( (Byte*) this + 0x8c);
	LOG ("  Port: %04x, Server port: %04x\n", myPort, id);
	BindComplete (noErr);
}

void TRFCOMMTool::ConnectStart ()
{
	BluntConnectionCommand* command = new BluntConnectionCommand ();
	
	LOG ("TRFCOMMTool::ConnectStart\n");
	command->fToolPort = * (TObjectId*) ( (Byte*) this + 0x8c);
	memcpy (command->fBdAddr, fPeerBdAddr, 6);
	command->fTargetLayer = L_RFCOMM;
	command->fL2CAPProtocol = PSM_RFCOMM;
	command->fRFCOMMPort = fPeerRFCOMMPort;
	fServerPort.Send (command, sizeof (*command), kNoTimeout, M_COMMAND);

//	memcpy (command->fBdAddr, "\x91\x1B\x29\x60\x10\x00", 6); // BlueZ
//	memcpy (command->fBdAddr, "\xa6\x62\x02\x57\x60\x00", 6); // Mac
}

void TRFCOMMTool::ListenStart (void)
{
	LOG ("TRFCOMMTool::ListenStart\n");
	ListenComplete (noErr);
}

void TRFCOMMTool::AcceptStart (void)
{
	LOG ("TRFCOMMTool::AcceptStart\n");
	AcceptComplete (noErr);
}

void TRFCOMMTool::TerminateConnection (void)
{
	LOG ("TRFCOMMTool::TerminateConnection\n");
	BluntDisconnectCommand* command = new BluntDisconnectCommand ();
	
	command->fHCIHandle = fHCIHandle;
	fServerPort.Send (command, sizeof (*command), kNoTimeout, M_COMMAND);
	TCommTool::TerminateConnection ();
}

void TRFCOMMTool::TerminateComplete (void)
{
	LOG ("TRFCOMMTool::TerminateComplete\n");
//	TCommTool::TerminateConnection ();
}

#pragma mark -

NewtonErr TRFCOMMTool::PutBytes (CBufferList *)
{
	LOG ("TRFCOMMTool::PutBytes\n");
	return noErr;
}

NewtonErr TRFCOMMTool::PutFramedBytes (CBufferList *, Boolean)
{
	LOG ("TRFCOMMTool::PutFramedBytes\n");
	return noErr;
}

void TRFCOMMTool::KillPut (void)
{
	LOG ("TRFCOMMTool::KillPut\n");
}

NewtonErr TRFCOMMTool::GetBytes (CBufferList *)
{
	LOG ("TRFCOMMTool::GetBytes\n");
	return noErr;
}

NewtonErr TRFCOMMTool::GetFramedBytes (CBufferList *)
{
	LOG ("TRFCOMMTool::GetFramedBytes\n");
	return noErr;
}

void TRFCOMMTool::KillGet (void)
{
	LOG ("TRFCOMMTool::KillGet\n");
}

