#ifndef __SDPPLAYER_H
#define __SDPPLAYER_H

class L2CAP;
class Channel;

#define UUID_SDP									0x0001
#define UUID_RFCOMM									0x0003
#define UUID_OBEX									0x0008
#define UUID_L2CAP									0x0100
#define UUID_SERIAL_PORT							0x1101
#define UUID_PPP_LAN								0x1102
#define UUID_DUN									0x1103
#define UUID_OBEX_PUSH								0x1105
#define UUID_OBEX_FILETRANSFER						0x1106

class SDP: public Handler
{
public:
	int						fState;

	UByte					*fInputPacket;
	Short					fInputPacketLength;
	Short					fLocalTID;
	Short					fRemoteTID;
	Byte					fPDUID;
	
	int						fCurrentQueriedService;

							SDP (void);
							
	void					Transition (ULong event, void *eventData = NULL);
	int						Action (int state, void *eventData = NULL);
	Boolean					HandleData (UByte *data, ULong length);
	void					HandleTimer (void *userData);
	
	void					Connect ();
	
	void					GetServices (void);
	
	// Utility functions

	void					GetDataElementLength (UByte *data, ULong& length, Byte& fieldLength);	
	void					GetSequenceLength (UByte *data, ULong& length, Byte& fieldLength);
	void					GetUUIDFromSequence (UByte *data, ULong& UUID, Byte& fieldLength);
	void					GetAttrRangeFromSequence (UByte *data, ULong& attr, Byte& fieldLength);
	void					GetInteger (UByte *data, Long& value, Byte& fieldLength);
	
	// Events

	void					ErrorResponse (void);
	void					ServiceSearchRequest (void);
	void					ServiceSearchResponse (void);
	void					ServiceAttributeRequest (void);
	void					ServiceAttributeResponse (void);
	void					ServiceSearchAttributeRequest (void);
	void					ServiceSearchAttributeResponse (void);

	// Actions

	void					SndErrorResponse (void);
	void					SndServiceSearchRequest (void);
	void					SndServiceSearchResponse (ULong *UUID, Byte count);
	void					SndServiceAttributeRequest (void);
	void					SndServiceAttributeResponse (ULong handle);
	void					SndServiceSearchAttributeRequest (ULong UUID);
	void					SndServiceSearchAttributeResponse (ULong *UUID, Byte UUIDCount, ULong *attr, Byte attrCount);
	
	void					SndData (UByte *data, Short length);
};

#endif